#
# $Id: NewsletterTheme.py 105 2006-01-11 09:49:46Z zope $
#

"""NewsletterCentral class"""

# Standard Python imports
import string, re
import random
import commands
import smtplib
import csv
import os

# Zope core imports
from Globals import InitializeClass
from AccessControl import Permissions, getSecurityManager, ClassSecurityInfo, Unauthorized
from AccessControl.SecurityManagement import newSecurityManager, setSecurityManager
from OFS import Folder
from Products.PageTemplates import Expressions
from Products.PageTemplates.TALES import CompilerError
from Persistence import Persistent

# CMF/Plone imports
from Products.CMFCore.CMFCorePermissions import View, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.CMFCatalogAware import CMFCatalogAware
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault import SkinnedFolder
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

# Product specific imports
from PNLPermissions import *
from PNLUtils import *
from PNLBase import PNLContentBase

DEFAULT_ACTIVATION_TEMPLATE = """Dear subscriber,

We received and pre-recorded your newsletter subscription.
You must now activate your account to receive our newsletters.
To do this, just browse to that URL...
%(url)s
Then you'll receive our next newsletters at %(email)s

PLEASE DON'T REPLY TO THIS MAIL"""
# NewsletterTheme definition

DEFAULT_NEWSLETTER_FOOTER = """You received this newsletter because you subscribed it.<br />
You may change your preferences at ...<br />
<a href="%(url)s">%(url)s</a>
"""

#################
## The factory ##
#################
def addNewsletterTheme(self, id, title = '', REQUEST = {}):
    """
    Factory method for a NewsletterTheme object
    """
    obj = NewsletterTheme(id, title)
    self._setObject(id, obj)
    getattr(self, id)._post_init()
    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')
    return

########################################
## The NewsletterCentral content type ##
########################################

class NewsletterTheme(SkinnedFolder.SkinnedFolder, DefaultDublinCoreImpl, PNLContentBase):
    """NewsletterTheme class
    Instance attributes:

    o id, title: CMF/Plone standards

    o body: the html text that describes that newsletter central

    o format_list : List of available format for newsletter

    o default_format : default format for newsletters

    o _internalVersion: provision for future upgrades

    o _subscribersCount: counter incremented for each new subsciber

    o testEmail: mail address for testings (defaults to the member's mail)

    o authorEmail: mail address for broadcasting

    o activationMailSubject: the subject of the activation e-mail

    o activationMailTemplate: the content model of the activation e-mail

    o notify: echoes new subscriptions to testEmail

    o renderTemplate: TALES expression that returns the rendering template

    o _v_renderTemplate: compiled version of above

    o extraRecipients: TALES expression for additional recipients

    o _v_extraRecipients: compiled version of above
    """

    ########################################
    ## Registration info for portal_types ##
    ########################################

    factory_type_information = {
        'id': 'NewsletterTheme',
        'portal_type': 'NewsletterTheme',
        'meta_type': 'NewsletterTheme',
        'description': 'Manage your newsletters with this',
        'content_icon': 'NewsletterTheme.gif',
        'product': 'PloneGazette',
        'factory': 'addNewsletterTheme',
        'immediate_view': 'folder_listing',
        'filter_content_types': 1,
        'allowed_content_types': ('Newsletter', 'Subscriber', 'NewsletterBTree'),
        'actions': (
            {'id': 'view',
             'name': 'View',
             'action': 'string:${object_url}/NewsletterTheme_view',
             'permissions': (View,)},

            {'id': 'subscribe',
             'name': 'Subscribe',
             'action': 'string:${object_url}/NewsletterTheme_subscribeForm',
             'permissions': (View,)},

            {'id' : 'infos',
             'name' : 'Informations',
             'action' : 'string:${object_url}/subscribers_infos',
             'permissions' : (ChangeNewsletterTheme,)},

            {'id': 'folderlisting',
             'name': 'Folder Listing',
             'action': 'string:${object_url}/folder_contents',
             'permissions': (ChangeNewsletterTheme,)},

            {'id': 'edit',
             'name': 'Edit',
             'action': 'string:${object_url}/NewsletterTheme_editForm',
             'permissions': (ChangeNewsletterTheme,)},

            {'id': 'metadata',
             'name': 'Properties',
             'action': 'string:${object_url}/metadata_edit_form',
             'permissions': (ChangeNewsletterTheme,)},

            {'id' : 'NewsletterTheme_importForm',
             'name' : 'Import',
             'action' : 'string:${object_url}/NewsletterTheme_importForm',
             'permissions' : (ChangeNewsletterTheme,)},
            ),
        }

    ###########################
    ## Basic class behaviour ##
    ###########################

    meta_type = factory_type_information['meta_type']

    manage_options = Folder.Folder.manage_options + CMFCatalogAware.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    _csv_import_log = ''

    # Dummy permission instanciation.
    # Permissions has to be "dummily" instanciated in order to be reachable for the factory_type_information structure.
    # In order to be very clear, we just declare each and every permission we use in this product.
    security.declareProtected(ChangeNewsletterTheme, "dummyMethod_editPermission")

    # Init method
    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        # NOTA : We shouldn't call parent's __init__ method as it would link to PortalFolder.__init__ and this
        # method sets 'self.id' and 'self.title' which is unuseful for us.

        global DEFAULT_ACTIVATION_TEMPLATE
        DefaultDublinCoreImpl.__init__(self)
        self._internalVersion = 2
        self._subscribersCount = 0
        self.id = id
        self.format_list = ['HTML', 'Text']
        self.default_format = 'HTML'
        self.title = title
        self.description = ''
        self.testEmail = ''
        self.authorEmail = ''
        self.replyto = ''
        self.activationMailSubject = "Please activate your newsletter account"
        self.activationMailTemplate = DEFAULT_ACTIVATION_TEMPLATE
        self.newsletterFooter = DEFAULT_NEWSLETTER_FOOTER
        self.notify = False
        self.renderTemplate = ''
        self._v_renderTemplate = None
        self.extraRecipients = ''
        self._v_extraRecipients = None
        self.lynxCommand = ''
        self.lynxOptions = ''
        self.subscriber_folder_id = ''
        return


    security.declarePrivate('_post_init')
    def _post_init(self):
        """Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """

        self.indexObject()
        return

    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' or metadatas.
    security.declareProtected(ChangeNewsletterTheme, 'manage_editNewsletterTheme')
    def manage_editNewsletterTheme(self, title = '', REQUEST = {}):
        """
        manage_editNewsletterTheme(self, title = '', REQUEST = {}) => object modification method
        """

        translate = self.translate
        # Change title
        self.title = title
        if REQUEST is not None:
            return self.folder_contents(self, REQUEST, portal_status_message=translate('Updated folder.', domain='plonegazette'))
        return

    security.declareProtected(ChangeNewsletterTheme, 'edit')
    def edit(self, title='',  default_format='', testEmail='', authorEmail='', replyto='',
             activationMailSubject='', activationMailTemplate='', newsletterFooter='', notify=False,
             renderTemplate='', extraRecipients='', lynxCommand='', subscriber_folder_id='', lynxOptions = ''):
        """Changes values"""

        self.title = title.strip()
        self.default_format = default_format
        self.testEmail = testEmail.strip()
        self.authorEmail = authorEmail.strip()
        self.replyto = replyto.strip()
        self.activationMailSubject = activationMailSubject.strip()
        self.activationMailTemplate = activationMailTemplate.strip()
        self.newsletterFooter = newsletterFooter.strip()
        self.notify = notify
        self.renderTemplate = renderTemplate.strip()
        self.extraRecipients = extraRecipients.strip()
        dummy = self.getRenderTemplate(recompile=1)
        dummy = self.getExtraRecipients(recompile=1)
        self.lynxCommand = lynxCommand
        self.lynxOptions = lynxOptions
        self.reindexObject()
        self.subscriber_folder_id = subscriber_folder_id
        return

    security.declareProtected(View, 'getNewsletters')
    def getNewsletters(self):
        """
        Return the "Newsletter" objects
        """
        mtool = getToolByName(self, 'portal_membership')
        checkPermission = mtool.checkPermission
        result = [ x for x in self.objectValues('Newsletter') if checkPermission('View', x)]
        return result

    # This method comes from PloneFolder class.

    security.declareProtected(ListFolderContents, 'listFolderContents')
    def listFolderContents( self, spec=None, contentFilter=None, suppressHiddenFiles=0 ):
        """
        Hook around 'contentValues' to let 'folder_contents'
        be protected.  Duplicating skip_unauthorized behavior of dtml-in.

        In the world of Plone we do not want to show objects that begin with a .
        So we have added a simply check.  We probably dont want to raise an
        Exception as much as we want to not show it.

        """

        items = self.contentValues(spec=spec, filter=contentFilter)
        l = []
        for obj in items:
            id = obj.getId()
            v = obj
            try:
                if suppressHiddenFiles and id[:1]=='.':
                    raise Unauthorized(id, v)
                if getSecurityManager().validate(self, self, id, v):
                    l.append(obj)
            except (Unauthorized, 'Unauthorized'):
                pass
        return l

    security.declarePublic('createSubscriberObject')
    def createSubscriberObject(self, id):
        """
        """

        target_folder = self.getSubscriberFolder()
        if target_folder is None:
            target_folder = self

        target_folder.invokeFactory('Subscriber', id)
        obj = getattr(target_folder, id)
        return obj

    security.declarePublic('getSubscriberFolder')
    def getSubscriberFolder(self):
        """
        Return the folder where subscriber objects will be create.
        It will be a "NewsletterBTree" if it exist, or the "NewsletterTheme" itself.
        """
        target_folder = None
        subscriber_folder_id = self.subscriber_folder_id
        if subscriber_folder_id and subscriber_folder_id in self.objectIds('NewsletterBTree'):
            target_folder = getattr(self, subscriber_folder_id)

        return target_folder

    security.declarePublic('getSubscriberById')
    def getSubscriberById(self, id):
        """
        """
        subscriber = None

        subscriber_folder = self.getSubscriberFolder()
        if subscriber_folder is not None:
            if id in subscriber_folder.objectIds('Subscriber'):
                subscriber = getattr(subscriber_folder, id)

        if subscriber is None:
            if id in self.objectIds('Subscriber'):
                subscriber = getattr(self, id)

        return subscriber

    ##################
    ## UI handlings ##
    ##################

    security.declarePrivate('_getRandomIdForSubscriber')
    def _getRandomIdForSubscriber(self):
        """
        """
        validChars = [c for c in string.letters + string.digits if ord(c) < 128]
        newId = "%05d" % self._subscribersCount + ''.join([random.choice(validChars) for x in range(5)])
        return newId

    security.declarePublic('subscribeFormProcess')
    def subscribeFormProcess(self, REQUEST=None):
        """Handles NewsletterTheme_subscribeForm"""

        translate = self.translate
        if REQUEST is None:
            REQUEST = self.REQUEST
        errors = {}
        data = {}
        if REQUEST.form.has_key('email'):
            # Form submitted
            translate = self.translate
            email = REQUEST.form.get('email', '').strip()
            data['email'] = email
            if not checkMailAddress(email):
                errors['email'] = translate('This is not a valid mail address', domain='plonegazette')
            format = REQUEST.form.get('format', self.default_format)
            data['format'] = format
            if self.alreadySubscriber(email):
                errors['email'] = translate('There is already a subscriber with this address', domain='plonegazette')
            if not errors:
                # Creating the new account
                self._subscribersCount += 1
                newId = self._getRandomIdForSubscriber()
                # Continue method as owner of this object for "invokeFactory" security checkings.
                # (creating new objects through invokeFactory is not possible as anonymous because an owner is required)
                oldSecurityManager = getSecurityManager()
                newSecurityManager(REQUEST, ownerOfObject(self))
                newSubscriber = self.createSubscriberObject(newId)
                newSubscriber.edit(format=format, active=0, title=email)
                try:
                    self.portal_workflow.doActionFor(newSubscriber,'publish')
                except:
                    pass
                setSecurityManager(oldSecurityManager)

                # Make URL for editing this object
                actions_tool = getToolByName(self, 'portal_actions')
                actions = actions_tool.listFilteredActionsFor(object=newSubscriber)
                subscriberEditUrl = [action['url'] for action in actions['object']
                                     if action['id'] == 'edit'][0]

                # Make and send the activation mail
                mailBody = ("From: %s\r\n"
                            "To: %s\r\n"
                            "Content-Type: text/plain; charset=%s\r\n"
                            "Subject: %s\r\n\r\n")
                mailBody = mailBody % (self.authorEmail, data['email'],
                                       self.ploneCharset(), self.activationMailSubject)
                mailBody += self.activationMailTemplate % {'url': self.absolute_url() + '?active=%s&format=%s' % (newId, format), 'email': email}
                smtp = self.getSmtp()
                smtp.sendmail(self.authorEmail, (email,), mailBody)
                if self.notify:
                    # Notify the NewsletterTheme owner
                    mailBody = ("From: %s\r\n"
                                "To: %s\r\n"
                                "Content-Type: text/plain; charset=%s\r\n"
                                "Subject: %s : %s\r\n\r\n"
                                "%s\n%s")
                    mailBody = mailBody % (self.authorEmail, self.testEmail,
                                           self.ploneCharset(),
                                           self.title,
                                           translate("Newsletter new subscriber", domain='plonegazette'),
                                           translate("See the new account at...", domain='plonegazette'),
                                           subscriberEditUrl)
                    smtp.sendmail(self.authorEmail, (self.testEmail,), mailBody)
                smtp.close()
                data['success'] = 1
        else:
            # First entry in the form
            data['email'] = ''
            data['format'] = self.default_format
        return data, errors

    security.declarePublic('unSubscribe')
    def unSubscribe(self, subscriber_id, REQUEST=None):
        """The subscriber clicked the Unsubscribe button
        """
        subscriber = self.getSubscriberById(subscriber_id)
        if subscriber is not None:
            parent = subscriber.aq_parent
            parent.manage_delObjects([subscriber_id,])

        newSecurityManager(REQUEST, ownerOfObject(self))
        if REQUEST is not None:
            REQUEST.RESPONSE.redirect(self.absolute_url() + '/NewsletterTheme_unsubscribed')
        return

    def checkMailAddress(self, mail):
        return  checkMailAddress(mail)


    def checkTalExpressions(self, talExpressions, state=None, errors=None):
        """
        Check TALES expressions for edit_form validation
        """
        translate = self.translate
        talEngine = Expressions.getEngine()
        for fieldId, value in talExpressions:
            if not value:
                continue
            try:
                dummy = talEngine.compile(value)
            except CompilerError, e:
                msg = translate('TALES compilation error: ', domain='plonegazette') + str(e)
                if state is not None:
                    state.setError(fieldId, msg)
                if errors is not None:
                    errors[fieldId] = msg
        if state is not None:
            return state
        elif errors is not None:
            return errors
        return

    def checkLynxCommandFromEditForm(self, lynxCommand, lynxOptions, state=None, errors=None):
        translate = self.translate
        if not self.checkLynx(command=lynxCommand, options=lynxOptions):
            msg = translate('Please ask the system administrator the command that runs Lynx', domain='plonegazette')
            if errors is not None:
                errors['lynxCommand'] = msg
            if state is not None:
                state.setError('lynxCommand', msg)
        if state is not None:
            return state
        elif errors is not None:
            return errors
        return

    ##############
    ## Utilities #
    ##############

    security.declarePublic('alreadySubscriber')
    def alreadySubscriber(self, email):
        """Checks wether email is in the subscribers
        """

        # check for subscribers inside "NewsletterTheme"
        all_subscriber_ids = list(self.objectIds('Subscriber'))
        for id in all_subscriber_ids:
            obj = getattr(self, id)
            obj_email = obj.title
            if email == obj_email:
                return 1

        # check for subscribers inside "Subscriber NewsletterBTree" if exist
        subscriber_folder = self.getSubscriberFolder()
        if subscriber_folder is not None:
            all_subscriber_ids = subscriber_folder.objectIds('Subscriber')
            for id in all_subscriber_ids:
                obj = getattr(subscriber_folder, id)
                obj_email = obj.title
                if email == obj_email:
                    return 1

        return 0

    security.declarePublic('checkLynx')
    def checkLynx(self, command='', options=''):
        """Guess if the above "LYNX_COMMAND" is valid
        returns True if valid"""

        if not command:
            command = self.lynxCommand
        if not command:
            command = LYNX_COMMAND
        try:
            if not options:
                options = self.lynxOptions
        except:
            pass
        if not options:
            options = LYNX_OPTIONS
        command = command + " " + options + " " + self.absolute_url() + "/../testLynx"
        status, output = commands.getstatusoutput(command)
        if status:
            return False
        return "Lynx test OK" in output

    security.declareProtected(ChangeNewsletterTheme, 'getSmtp')
    def getSmtp(self):
        """Returns a usable SMTP server"""

        mh = self.MailHost
        return smtplib.SMTP(mh.smtp_host or 'localhost', mh.smtp_port)

    security.declarePublic('getRenderTemplate')
    def getRenderTemplate(self, recompile=0):
        """Returns the template that renders the HTML newsletter
        recompile=1 -> recompile TAL expression"""

        talEngine = Expressions.getEngine()
        try:
            dummy = self._v_renderTemplate
        except AttributeError, e:
            recompile = 1
        if recompile:
            if self.renderTemplate:
                self._v_renderTemplate = talEngine.compile(self.renderTemplate)
            else:
                self._v_renderTemplate = None
        if self._v_renderTemplate:
            data = {'here': self}
            template = self._v_renderTemplate(talEngine.getContext(data))
        else:
            template = getattr(self, 'newsletter_mua_formatter')
        return template

    security.declarePublic('getExtraRecipients')
    def getExtraRecipients(self, recompile=0):
        """Provides a TALES compiled extra list of recipients
        recompile=1 -> recompile TAL expression"""

        talEngine = Expressions.getEngine()
        try:
            dummy = self._v_extraRecipients
        except AttributeError, e:
            recompile = 1
        if recompile:
            # Not used since last cache extraction
            if self.extraRecipients:
                self._v_extraRecipients = talEngine.compile(self.extraRecipients)
            else:
                self._v_extraRecipients = None
        if self._v_extraRecipients:
            data = {'here': self}
            return self._v_extraRecipients(talEngine.getContext(data))()
        else:
            return []

    security.declareProtected(ChangeNewsletterTheme, 'mailingInfos')
    def mailingInfos(self):
        """Return mailing info from all subscribers"""

        subscribersIds = self.objectIds('Subscriber')
        mi = []
        for id in subscribersIds:
            subscriber = getattr(self, id)
            si = subscriber.mailingInfo()
            if si:
                mi.append(si)

        subscriber_folder = self.getSubscriberFolder()
        if subscriber_folder is not None:
            subscribersIds = subscriber_folder.objectIds('Subscriber')
            for id in subscribersIds:
                subscriber = getattr(subscriber_folder, id)
                si = subscriber.mailingInfo()
                if si:
                    mi.append(si)


        return mi

    security.declarePrivate('_getStatsForSubscriber')
    def _getStatsForSubscriber(self, subscriber, stats, listing):
        """
        """
        format = subscriber.format
        active = subscriber.active
        if format == 'HTML':
            if active:
                stats['htmlactive'] += 1
            else:
                stats['htmlinactive'] += 1
        else:
            if active:
                stats['plaintextactive'] += 1
            else:
                stats['plaintextinactive'] += 1

        listing.append({'title' : subscriber.title, 'url' : subscriber.absolute_url(), 'active' : active, 'format' : format})

        return (stats, listing)

    security.declareProtected(ChangeNewsletterTheme, 'subscriberStats')
    def subscriberStats(self):
        """Returns a dict with statistics about subscribers"""

        subscribersIds = self.objectIds('Subscriber')
        listing = []
        stats = {'total': len(subscribersIds),
                 'htmlactive': 0,
                 'plaintextactive': 0,
                 'htmlinactive': 0,
                 'plaintextinactive': 0,
                 'totalhtml': 0,
                 'totalplaintext': 0,
                 'totalactive': 0,
                 'totalinactive': 0}

        for id in subscribersIds:
            subscriber = getattr(self, id)
            stats, listing = self._getStatsForSubscriber(subscriber, stats, listing)

        subscriber_folder = self.getSubscriberFolder()
        if subscriber_folder is not None:
            subscribersIds = subscriber_folder.objectIds('Subscriber')
            stats['total'] = stats['total'] + len(subscribersIds)
            for id in subscribersIds:
                subscriber = getattr(subscriber_folder, id)
                stats, listing = self._getStatsForSubscriber(subscriber, stats, listing)

        stats['totalhtml'] = stats['htmlactive'] + stats['htmlinactive']
        stats['totalplaintext'] = stats['plaintextactive'] + stats['plaintextinactive']
        stats['totalactive'] = stats['htmlactive'] + stats['plaintextactive']
        stats['totalinactive'] = stats['htmlinactive'] + stats['plaintextinactive']
        return (stats, listing)

    security.declareProtected(ChangeNewsletterTheme, 'extraRecipientStats')
    def extraRecipientStats(self):
        """Returns a dict with statistics about extra recipients"""
        extraRecipients = self.getExtraRecipients()
        stats = {'total': len(extraRecipients),
                 'html': 0,
                 'plaintext': 0}
        for mail, html, editurl in extraRecipients:
            if html:
                stats['html'] += 1
            else:
                stats['plaintext'] += 1
        return stats

    ############################
    ## portal_catalog support ##
    ############################

    security.declarePublic('SearchableText')
    def SearchableText(self):
        return self.title

    ##########################
    ## CSV import features
    #########################

    security.declareProtected(ChangeNewsletterTheme, 'createSubscribersFromCSV')
    def createSubscribersFromCSV(self, file_upload):
        """
        Create all subscribers objects from csv file uploaded
        """

        # Reset old log
        self._csv_import_log = ''

        filename = file_upload.filename
        filedatas = file_upload.read()
        var_directory = self.Control_Panel.getCLIENT_HOME()

        dialect = csv.excel

        # first, create a temp file on file system
        self._createTempFile(var_directory, filename, filedatas)

        # open temp csv file
        reader = csv.DictReader(open('%s/%s' % (var_directory, filename)), ['email', 'active', 'format'], dialect=dialect)

        # get target folder for subscribers object, or create it if not exist
        subscriber_folder = self.getSubscriberFolder()
        if subscriber_folder is None:
            self.invokeFactory('NewsletterBTree', 'subscribers')
            self.subscriber_folder_id = 'subscribers'
            subscriber_folder = getattr(self, 'subscribers')

        # remove headers
        first_row = reader.next()
        if first_row['email']!='email':
            return "You must add headers to the csv file : email, active, format ('email' at least)"


        # for each row, create a subscriber object
        default_format = self.default_format
        k = 0
        already_used = []
        not_valid = []
        for row in reader:

            # get the field value, or the default value
            if row['active']=='1':
                active = True
            elif row['active']=='0':
                active = False
            else:
                active = False

            if row['format']:
                if row['format'].lower()=='html':
                    format = 'HTML'
                elif row['format'].lower()=='text':
                    format = 'Text'
                else:
                    format = default_format
            else:
                format = default_format

            email = row['email']
            email = email.strip()

            # check mail address validity
            if not checkMailAddress(email):
                not_valid.append(email)
            else:

                # check if subscriber already exist
                if self.alreadySubscriber(email):
                    already_used.append(email)
                else:
                    newId = self._getRandomIdForSubscriber()
                    subscriber = self.createSubscriberObject(newId)

                    subscriber.edit(format=format, active=active, title=email)
                    k += 1
                    self._subscribersCount += 1

        # remove temp csv file
        os.remove('%s/%s' % (var_directory, filename))

        self._logCSVImportResult(not_valid, already_used)

        msg = ''
        if k:
            msg += '%s subcribers created. ' % str(k)
        if len(already_used):
            msg += '%s users were already subscriber on this newsletter theme. ' % len(already_used)

        if len(not_valid):
            msg += '%s emails were not valid. ' % len(not_valid)

        return msg

    security.declareProtected(ChangeNewsletterTheme, 'getCSVImportLog')
    def getCSVImportLogs(self):
        """
        """
        return self._csv_import_log.strip()

    security.declarePrivate('_logCSVImportResult')
    def _logCSVImportResult(self, not_valid, already_used):
        """
        """
        result = ''
        if len(already_used):
            result += '<h2>Already subscribed</h2>'
            result += '<p>'
            result += '<br />'.join(already_used)
            result += '</p>'
        if len(not_valid):
            result += '<h2>Not valid emails</h2>'
            result += '<p>'
            result += '<br />'.join(not_valid)
            result += '</p>'
        self._csv_import_log = result
        return

    security.declarePrivate('_createTempFile')
    def _createTempFile(self, home, filename, filedatas):
        """
        Create a temp file inside zope "var" directory
        """
        temp_file = open('%s/%s' % (home, filename), 'w')
        temp_file.write(filedatas)
        temp_file.close()
        return

# Class instanciation
InitializeClass(NewsletterTheme)

